from genie_pkg.generators import *
import re
import pytest


def test_ipv4():
    ipv4 = generate_ip()
    assert re.match(r'\d+\.\d+\.\d+\.\d+', ipv4)


def test_ipv6():
    ipv6 = generate_ip(v=6)
    v6_part_regex = r'[a-z0-9]{2,4}'
    assert re.match(
        '{0}:{0}:{0}:{0}:{0}:{0}:{0}:{0}'.format(v6_part_regex), ipv6)



def test_bad_ip_length():
    with pytest.raises(GenieException) as _:
        generate_ip(3)

def test_ipv4_subnet():
    # network containing all addresses from 10.0.0.0 to 10.0.0.255
    ipv4 = generate_ipv4_in_subnet("10.0.0.0/24")
    assert re.match(r'10\.0\.0\.[0-9]{1,3}', ipv4)


def test_random_geo():
    for i in range(5):
        x0, y0 = random_geo_coords(radius=1000)
        assert math.isclose(x0, -37.814, abs_tol=0.010)
        assert math.isclose(y0, 144.963, abs_tol=0.010)


def test_check_digit_luhn_mod_10_mc():
    assert check_digit_luhn_mod_10('7992739871') == 3
    assert check_digit_luhn_mod_10('536990340067168') == 0
    assert check_digit_luhn_mod_10('540436862055838') == 9
    assert check_digit_luhn_mod_10('532420806394835') == 7
    assert check_digit_luhn_mod_10('532420499852544') == 4


def test_check_digit_luhn_mod_10_visa():
    assert check_digit_luhn_mod_10('402400714020726') == 6
    assert check_digit_luhn_mod_10('433696811345131') == 9
    assert check_digit_luhn_mod_10('455604196816395') == 0
    assert check_digit_luhn_mod_10('453260352189118') == 4


def test_random_mastercard_number():
    assert random_mastercard_number() is not None


def test_random_visacard_number():
    cc = random_visacard_number(13)
    assert len(cc) == 13


def test_one_of():
    choices = [1, 2, 3]
    v = one_of(choices)
    assert v in choices


def test_one_of_throws_exception():
    with pytest.raises(Exception) as e_info:
        one_of('blah')
        assert False


def test_random_wonderland_text():
    text = random_wonderland_text(2)
    #funny regex may not work for all sentences. No use installing nltk for a test, hence no len assertion
    assert text is not None


def test_random_dob():
    assert random_dob(year=2019, month=1, day=1) == '01/01/2019'
    feb_born = random_dob(year=2019, month=2)
    assert int(feb_born.split('/')[1]) <= 28
    nonfeb_born = random_dob(year=2019, month=3)
    assert int(nonfeb_born.split('/')[1]) <= 30
    assert random_dob() is not None


def test_utc_epoch_start_and_end_ms_for():
    s, e = utc_epoch_start_and_end_ms_for(2019, 7, 22)
    ms_in_day = e - s
    assert ms_in_day + 10 == 24 * 60 * 60 * 1000

def test_utc_now_epoch():
    utc1 = utc_now_epoch_ms()
    utc2 = utc_now_epoch_ms()
    assert utc2 >= utc1


from genie_pkg.australia import Australia
import pytest
import math
from genie_pkg import GenieException

@pytest.fixture
def oz():
    return Australia()

def test_state(oz):
    assert oz.get_random_state() in ["VIC", "NSW", "ACT", "QLD", "SA", "NT", "TAS", "WA"]

def test_city_postcode(oz):
    locality, postcode = oz.get_random_city_postcode("VIC")
    assert int(postcode) >= 3000
    assert locality is not None

def test_city(oz):

    assert oz.get_city(state="VIC", postcode="3000") == "MELBOURNE"
    cities_3122 = [
                "HAWTHORN WEST",
                "HAWTHORN",
                "GLENFERRIE SOUTH",
                "HAWTHORN NORTH",
                "AUBURN SOUTH"]
    assert oz.get_city(state="VIC", postcode="3122") in cities_3122

def test_geo_coordinate(oz):
    x0, y0 = oz.get_random_geo_coordinate(state="VIC", postcode="3000")
    assert x0 is not None
    assert y0 is not None

def test_geo_coordinate_throws_exception_when_center_null(oz):
    with pytest.raises(GenieException) as e_info:
        oz.get_random_geo_coordinate(state="WA", postcode="6452")
        



import pytest
import random

from genie_pkg.fw_genie import generate, anonymise_columns

type_choices = ['int', 'float', 'str']
min_width = 5  # this is to make sure floats are valid


def test_fw():
    # Run all the tests in a loop to have better confidance of randomisation.
    for i in range(2, 5):
        ncols = random.randint(1, 10)
        default_specs = [(random.randint(min_width, 10), random.choice(type_choices),) for i in range(ncols)]
        colspecs = default_specs + [(10, 'date', '%d/%m/%Y', 2),
                                    (10, 'float', 3), 
                                    (15, 'email', 'mail.com'),
                                    (10, 'one_of', ['returnasis'])]
        data = generate(colspecs, nrows=1)
        expected_length = sum([c[0] for c in colspecs])
        for d in data:
            decoded = d.decode()
            assert len(decoded) == expected_length
            assert 'returnasis' in decoded
            assert 'mail.com' in decoded
            

def test_generate_bad_decode():
    # has to have atleast 1 special char for this test to work
    ncols = random.randint(1, 10)
    colspecs = [(random.randint(min_width, 10), random.choice(type_choices),) for i in range(ncols)]
    # run the test more than once
    bad_decodes = [
        ('windows-1252', 'utf-8'),
        ('utf-8', 'ascii'),
        ('utf-16', 'utf-8')
    ]
    
    for encoding, decoding in bad_decodes:
        for i in range(1, 3):
            for d in generate(colspecs + [(8, 'str')], nrows=1, encoding=encoding):
                try:
                    print(d.decode(decoding))
                    assert False
                except UnicodeDecodeError as e:
                    assert True


def test_generate_good_decode():
    ncols = random.randint(1, 10)
    colspecs = [(random.randint(min_width, 10), random.choice(type_choices),) for i in range(ncols)]
    good_decodes = [
        ('windows-1252', 'windows-1252'),
        ('utf-8', 'utf-8'),
        ('ascii', 'utf-8'),
    ]

    for encoding, decoding in good_decodes:
        for d in generate(colspecs + [(8, 'str')], nrows=1, encoding=encoding):
            try:
                print(d.decode(decoding))
                assert True
            except Exception as e:
                assert False


def test_anonymise():
    input_encoding = 'windows-1252'
    row = 'FReNG£Ni£iFthtR¥ubOswUPhmQWJoypvF¢MFcR'.encode(input_encoding)

    anonymous_col_specs = [(0, 5, 'int'), (28, 38, 'float')]
    anonymised = anonymise_columns(row, anonymous_col_specs, input_encoding)
    decoded = anonymised.decode(input_encoding)
    assert len(decoded) == len(row.decode(input_encoding))
    v = decoded[:5]
    assert v != 'FReNG'
    assert isinstance(int(v), int) == True


def test_bad_email_config_throws_exception():
    with pytest.raises(Exception) as e_info:
        row = 'FReNG£Ni£iFthtR¥ubOswUPhmQWJoypvF¢MFcR'.encode()

        anonymous_col_specs = [(28, 38, 'email', 'hotmail.com')]
        anonymise_columns(row, anonymous_col_specs)

def test_bad_date_length_throws_exception():
    with pytest.raises(Exception) as e_info:
        colspecs =  [(8, 'date')]
        for d in generate(colspecs, nrows=1):
            assert d is not None

def test_bad_float_length_throws_exception():
    with pytest.raises(Exception) as e_info:
        colspecs =  [(4, 'float', 3)]
        for d in generate(colspecs, nrows=1):
            assert d is not None


def test_bad_one_of_throws_exception_bad_length():
    with pytest.raises(Exception) as e_info:
        colspecs =  [(5, 'one_of', ['returnasis'])]
        for d in generate(colspecs, nrows=1):
            assert False

